#pragma once
#include <assert.h>
#include <string>
using namespace std;


class device {
public:
	string name;
	string *nodes;
	int *ndx;
	double x, y;
	double orientation;
	double value;
	device() { };
	device(int n);
	virtual ~device();
	virtual int getNodeCount() const = 0;
	virtual string getType() const = 0;
	virtual string getCode() const = 0;
	void parse(string tok[], int ntokens);
	void parse_value(string tok);
};

class Resistor: public device {
public:
	Resistor(): device(2) { }
	int getNodeCount() const { return 2; }
	string getType() const { return "resistor"; }
	string getCode() const { return "R"; }
};

class CurrentSource: public device {
public:
	CurrentSource(): device(2) { }
	int getNodeCount() const { return 2; }
	string getType()const { return "current source"; }
	string getCode() const { return "I"; }
};


class circuit {
public:
	string *nodes;
	int ncount;
	device **element;
	int ndevices;
	circuit() {create(8,16); }
	~circuit();
	void create(int node_count, int device_count);
	void read(FILE *in);
	void list_nodes();
	void list_devices();
	void build_node_list();
	int add_node(string str);
};

